from datetime import date, time, timedelta

from factory import Sequence, post_generation
from factory.django import DjangoModelFactory

from django.contrib.auth.models import Permission, User
from django.db.models import QuerySet
from django.utils.timezone import now
from program.models import (
    CBA,
    Category,
    FundingCategory,
    Image,
    Language,
    License,
    LinkType,
    MusicFocus,
    Note,
    Playlist,
    PlaylistEntry,
    Profile,
    RadioSettings,
    RRule,
    Schedule,
    Show,
    TimeSlot,
    Topic,
    Type,
)


class CommonUserFactory(DjangoModelFactory):
    class Meta:
        model = User

    password = "password"
    username = Sequence(lambda n: "common_%d" % n)


class UserWithPermissionsFactory(DjangoModelFactory):
    class Meta:
        model = User

    username = Sequence(lambda n: "user_with_perm%d" % n)

    @post_generation
    def user_permissions(self, create, extracted, **kwargs):
        if not create:
            return

        if extracted:
            if isinstance(extracted, Permission):
                self.user_permissions.add(extracted)

            if isinstance(extracted, QuerySet):
                for permission in extracted:
                    self.user_permissions.add(permission)


class ProfileFactory(DjangoModelFactory):
    class Meta:
        model = Profile

    name = Sequence(lambda n: "profile %d" % n)


class ImageFactory(DjangoModelFactory):
    class Meta:
        model = Image


class FundingCategoryFactory(DjangoModelFactory):
    class Meta:
        model = FundingCategory

    name = Sequence(lambda n: "funding category %d" % n)
    slug = Sequence(lambda n: "funding-category_%d" % n)


class TypeFactory(DjangoModelFactory):
    class Meta:
        model = Type

    name = Sequence(lambda n: "type %d" % n)
    slug = Sequence(lambda n: "type_%d" % n)


class ShowFactory(DjangoModelFactory):
    class Meta:
        model = Show

    name = Sequence(lambda n: "show %d" % n)
    slug = Sequence(lambda n: "show_%d" % n)


class RRuleFactory(DjangoModelFactory):
    class Meta:
        model = RRule


class ScheduleFactory(DjangoModelFactory):
    class Meta:
        model = Schedule

    end_time = time(3, 0, 0)
    first_date = date(2023, 1, 1)
    start_time = time(2, 0, 0)


class TimeslotFactory(DjangoModelFactory):
    class Meta:
        model = TimeSlot

    end = now() + timedelta(hours=1)
    start = now()


class LicenseFactory(DjangoModelFactory):
    class Meta:
        model = License

    identifier = Sequence(lambda n: "identifier %d" % n)
    name = Sequence(lambda n: "license %d" % n)


class NoteFactory(DjangoModelFactory):
    class Meta:
        model = Note


class CategoryFactory(DjangoModelFactory):
    class Meta:
        model = Category

    description = Sequence(lambda n: "description %d" % n)
    name = Sequence(lambda n: "category %d" % n)
    slug = Sequence(lambda n: "category_%d" % n)
    subtitle = Sequence(lambda n: "subtitle %d" % n)


class LanguageFactory(DjangoModelFactory):
    class Meta:
        model = Language

    name = Sequence(lambda n: "language %d" % n)


class LinkTypeFactory(DjangoModelFactory):
    class Meta:
        model = LinkType

    name = Sequence(lambda n: "language %d" % n)


class MusicFocusFactory(DjangoModelFactory):
    class Meta:
        model = MusicFocus

    name = Sequence(lambda n: "music focus %d" % n)
    slug = Sequence(lambda n: "music-focus_%d" % n)


class TopicFactory(DjangoModelFactory):
    class Meta:
        model = Topic

    name = Sequence(lambda n: "topic %d" % n)
    slug = Sequence(lambda n: "topic_%d" % n)


class OwnerFactory(DjangoModelFactory):
    class Meta:
        model = User


class RadioSettingsFactory(DjangoModelFactory):
    class Meta:
        model = RadioSettings


class CBAFactory(DjangoModelFactory):
    class Meta:
        model = CBA

    username = Sequence(lambda n: "username_%d" % n)
    user_token = Sequence(lambda n: "user_token_%d" % n)


class PlaylistFactory(DjangoModelFactory):
    class Meta:
        model = Playlist


class PlaylistEntryFactory(DjangoModelFactory):
    class Meta:
        model = PlaylistEntry

    duration = 73
    file_id = 42
    order = Sequence(lambda n: int(n))
    uri = f"file://0/{file_id}"
