import pytest

from program.tests.factories import LanguageFactory

pytestmark = pytest.mark.django_db


def url(language=None):
    return f"/api/v1/languages/{language.id}/" if language else "/api/v1/languages/"


def test_create_language(admin_api_client):
    data = {"is_active": True, "name": "NAME"}

    response = admin_api_client.post(url(), data, format="json")

    assert response.status_code == 201

    assert response.data["is_active"] == data["is_active"]
    assert response.data["name"] == data["name"]


def test_retrieve_langauge(common_api_client1, language):
    response = common_api_client1.get(url(language=language))

    assert response.status_code == 200

    assert response.data["is_active"] == language.is_active
    assert response.data["name"] == language.name


def test_delete_langauge(admin_api_client, language):
    response = admin_api_client.delete(url(language=language))

    assert response.status_code == 204


def test_list_langauges(common_api_client1):
    LANGUAGES = 3
    LanguageFactory.create_batch(size=LANGUAGES)

    response = common_api_client1.get(url())

    print(response.data)

    assert response.status_code == 200
    assert len(response.data) == LANGUAGES


def test_update_is_active(admin_api_client, language):
    update = {"is_active": False}

    response = admin_api_client.patch(url(language=language), update, format="json")

    assert response.status_code == 200
    assert response.data["is_active"] == update["is_active"]


def test_update_name(admin_api_client, language):
    update = {"name": "NAME"}

    response = admin_api_client.patch(url(language=language), update)

    assert response.status_code == 200
    assert response.data["name"] == update["name"]
